<?php
/*
Plugin Name: WP Autoload
Plugin URI: https://charles.lecklider.org/wp-plugin/wp-autoload
Description: Automatically load per-template JavaScript and CSS files.
Version: 1.1.0
Author: Charles Lecklider
Author URI: https://charles.lecklider.org/
License: GPL2
*/

/*  Copyright 2011  Charles Lecklider  (email : wordpress@charles.lecklider.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


class wp_autoload_options
{
	static $defaults	= array('js_dir'			=> 'js',
								'js_depends'		=> array(),
								'js_template_parts'	=> array(),
								'css_dir'			=> 'css');
}

if (is_admin()) {

	class wp_autoload
	{
		function __construct()
		{
			add_action('admin_init', array(&$this,'admin_init'));
			add_action('admin_menu', array(&$this,'admin_menu'));
		}

		function admin_init()
		{
			load_plugin_textdomain('wp-autoload', false, basename( dirname( __FILE__ ) ) . '/languages/' );

			register_setting('wp_autoload_options', 'wp_autoload', array(&$this,'validate'));

			/*** Scripts ***/
			add_settings_section('wp_autoload_scripts',			__('hdr_scripts','wp-autoload'),		array(&$this,'js'),					'wp_autoload');
			add_settings_field('wp_autoload_js_dir',			__('js_dir','wp-autoload'),				array(&$this,'js_dir'),				'wp_autoload', 'wp_autoload_scripts');
			add_settings_field('wp_autoload_js_depends',		__('js_depends','wp-autoload'),			array(&$this,'js_depends'),			'wp_autoload', 'wp_autoload_scripts');
			add_settings_field('wp_autoload_js_template_parts',	__('js_template_parts','wp-autoload'),	array(&$this,'js_template_parts'),	'wp_autoload', 'wp_autoload_scripts');

			/*** Styles ***/
			add_settings_section('wp_autoload_styles',	__('hdr_styles','wp-autoload'),	array(&$this,'css'),	'wp_autoload');
			add_settings_field('wp_autoload_css_dir',	__('css_dir','wp-autoload'),	array(&$this,'css_dir'),'wp_autoload', 'wp_autoload_styles');
		}

		function admin_menu()
		{
			add_options_page('WP Autoload','WP Autoload', 'manage_options', 'wp-autoload', array(&$this,'menu'));
		}

		function menu()
		{
			include 'wp-autoload-admin.php';
		}

		function validate($input)
		{
			$new_input = wp_autoload_options::$defaults;

			if ($input['js_dir'] == sanitize_title($input['js_dir'])) {
				$new_input['js_dir'] = $input['js_dir'];
			} else {
				add_settings_error('wp_autoload_js_dir', 'wp_autoload_js_dir_error', __('js_dir_err','wp-autoload'));
			}

			$depends_error = false;
			$js_depends = array();
			if (strlen($depends = trim($input['js_depends']))) {
				$depend = strtok($depends,',');
				while ($depend) {
					$tdepend = trim($depend);
					if ($tdepend == sanitize_title($tdepend)) {
						$js_depends[] = $tdepend;
					} else {
						$depends_error = true;
					}
					$depend = strtok($depends);
				}
			}
			$new_input['js_depends'] = $js_depends;
			if ($depends_error)
				add_settings_error('wp_autoload_js_depends', 'wp_autoload_js_depends_error', __('js_depends_err','wp-autoload'));

			if ($input['css_dir'] == sanitize_title($input['css_dir'])) {
				$new_input['css_dir'] = $input['css_dir'];
			} else {
				add_settings_error('wp_autoload_css_dir', 'wp_autoload_css_dir_error', __('css_dir_error','wp-autoload'));
			}

			$slugs = array_map(function($a){ return sanitize_title(basename($a,'.php')); }, glob(get_stylesheet_directory().'/*.php'));
			foreach($input['js_template_parts'] as $slug) {
				if (in_array($slug,$slugs))
					$new_input['js_template_parts'][] = $slug;
			}

			return $new_input;
		}

		function js()
		{
		//	_e('js_desc','wp-autoload');
		}

		function js_dir()
		{
			$options = get_option('wp_autoload',wp_autoload_options::$defaults);
			echo '<input class="medium-text" name="wp_autoload[js_dir]" type="text" value="'.$options['js_dir'].'" /> ';
			echo '<em>Directory ';
			echo (file_exists(get_stylesheet_directory().'/'.$options['js_dir'])) ? 'exists.' : 'doesn\'t exist.';
			echo '</em>';
		}

		function js_depends()
		{
			$options = get_option('wp_autoload',wp_autoload_options::$defaults);
			echo '<input class="regular-text" name="wp_autoload[js_depends]" type="text" value="'.implode(', ',$options['js_depends']).'" />';
		}

		function js_template_parts()
		{
			$excludes = array('404','search','taxonomy','category','tag','archive','paged','author','date','attachment','single','page','front-page','home','comments-popup','index','functions','comments','header','footer','sidebar');
			$options = get_option('wp_autoload',wp_autoload_options::$defaults);
			$files = array_map(function($a){ return basename($a,'.php'); }, glob(get_stylesheet_directory().'/*.php'));
			// exclude category templates
			foreach(get_categories(array('hide_empty'=>false)) as $category) {
				$excludes[] = 'category-'.$category->slug;
				$excludes[] = 'category-'.$category->term_id;
			}
			// exclude page templates
			foreach(get_pages() as $page) {
				$excludes[] = 'page-'.$page->post_name;
				$excludes[] = 'page-'.$page->ID;
			}
			// exclude custom post type templates
			foreach(get_post_types(array('public'=>true)) as $post_type) {
				$excludes[] = 'archive-'.$post_type;
				$excludes[] = 'single-'.$post_type;
			}
			// exclude taxonomy templates
			foreach(get_taxonomies('','names') as $taxonomy) {
				foreach(get_terms($taxonomy,array('hide_empty'=>false)) as $term) {
					$excludes[] = 'taxonomy-'.$taxonomy.'-'.$term->slug;
				}
				$excludes[] = 'taxonomy-'.$taxonomy;
			}
			foreach(array_diff($files,$excludes) as $file) {
				$slug		= sanitize_title($file);
				$input_id	= 'wp_autoload-js_template_parts-'.$slug;
				$checked	= (in_array($slug,$options['js_template_parts'])) ? 'checked="checked"' : '';
				echo '<input id="'.$input_id.'" name="wp_autoload[js_template_parts]['.$slug.']" type="checkbox" value="'.$slug.'" '.$checked.'/><label for="'.$input_id.'">'.$file.'</label><br/>';
			}
		}

		function css()
		{
		//	_e('css_desc','wp-autoload');
		}

		function css_dir()
		{
			 $options = get_option('wp_autoload',wp_autoload_options::$defaults);
			 echo '<input class="medium-text" name="wp_autoload[css_dir]" type="text" value="'.$options['css_dir'].'" />';
			 echo '<em>Directory ';
			 echo (file_exists(get_stylesheet_directory().'/'.$options['js_dir'])) ? 'exists.' : 'doesn\'t exist.';
			 echo '</em>';
		}
	}

} else {

	class wp_autoload
	{
		protected $wp_autoload_scripts	= array();

		function __construct()
		{
			add_filter('template_include',array(&$this,'template_include'));

			add_action('wp_footer',array(&$this,'footer'));

			/*** Sidebar ***/
			$this->template_include('sidebar.php');

			/*** Register actions for selected template parts ***/
			$options = get_option('wp_autoload',wp_autoload_options::$defaults);
			if (is_array($optons['js_template_parts'])) {
				foreach($optons['js_template_parts'] as $part) {
					add_action('get_template_part_'.$part, array(&$this,'template_part_scripts'), 10, 2);
				}
			}
		}

		function template_include($template)
		{
			 $options = get_option('wp_autoload_options',wp_autoload_options::$defaults);
			 $base = basename($template,'.php');

			 if (file_exists(get_template_directory().'/'.$options['js_dir'].'/'.$base.'.js')) {
				 wp_register_script($base,
									get_template_directory_uri().'/'.$options['js_dir'].'/'.$base.'.js',
									$options['js_depends'],
									'1.0',
									true);
				 wp_enqueue_script($base);
			 }
			 if (file_exists(get_template_directory().'/'.$options['css_dir'].'/'.$base.'.css')) {
				 wp_register_style($base,
								   get_template_directory_uri().'/'.$options['css_dir'].'/'.$base.'.css',
								   array(),
								   '1.0');
				 wp_enqueue_style($base);
			 }

			 return $template;
		}

		function template_part_scripts($slug, $name)
		{
			$options = get_option('wp_autoload_options',wp_autoload_options::$defaults);

			if (!array_key_exists($slug,$this->wp_autoload_scripts)) {
				$script = '/'.$options['js_dir'].'/'.$slug.'-'.$name.'.js';
				if ($this->wp_autoload_scripts[$slug] = file_exists(get_template_directory().$script)) {
					wp_register_script($slug,
									   get_template_directory_uri().$script,
									   $options['js_depends'],
									   '1.0',
									   true);
				}
			}
		}

		function footer()
		{
			foreach(array_keys($this->wp_autoload_scripts,true) as $script) {
				wp_print_scripts($script);
			}
		}

		function enqueue_script($script)
		{
			wp_enqueue_script($script);
			$this->wp_autoload_scripts[$script] = true;
		}
	}
}

$wp_autoload = new wp_autoload();

